@@ -11,11 +11,13 @@ use Scalar::Util qw/reftype/;
 use List::Util qw/first/;
 use HTML::Entities;
 use Encode qw/encode_utf8/;
+use DateTime;
+use Data::Visitor::Callback;
 
 use namespace::autoclean;
 
 # use *all* digits for fBSD ports
-our $VERSION = '3.06000';
+our $VERSION = '3.07000';
 
 $VERSION = eval $VERSION; # numify for warning-free dev releases
 
@@ -43,6 +45,8 @@ has _json_handler   => (
     handles => { _from_json => 'from_json' },
 );
 
+sub _synthetic_args { qw/authenticate since/ }
+
 sub BUILD {
     my $self = shift;
 
@@ -139,10 +143,10 @@ sub _decode_html_entities {
 
 # By default, Net::Twitter does not inflate objects, so just return the
 # hashref, untouched. This is really just a hook for Role::InflateObjects.
-sub _inflate_objects { return $_[1] }
+sub _inflate_objects { return $_[2] }
 
 sub _parse_result {
-    my ($self, $res) = @_;
+    my ($self, $res, $synthetic_args, $datetime_parser) = @_;
 
     # workaround for Laconica API returning bools as strings
     # (Fixed in Laconi.ca 0.7.4)
@@ -153,14 +157,19 @@ sub _parse_result {
     $self->_decode_html_entities($obj) if $obj && $self->decode_html_entities;
 
     # inflate the twitter object(s) if possible
-    $self->_inflate_objects($obj);
+    $self->_inflate_objects($datetime_parser, $obj);
 
     # Twitter sometimes returns an error with status code 200
     if ( ref $obj && reftype $obj eq 'HASH' && exists $obj->{error} ) {
         die Net::Twitter::Error->new(twitter_error => $obj, http_response => $res);
     }
 
-    return $obj if $res->is_success && defined $obj;
+    if  ( $res->is_success && defined $obj ) {
+        if ( my $since = delete $synthetic_args->{since} ) {
+            $self->_filter_since($datetime_parser, $obj, $since);
+        }
+        return $obj;
+    }
 
     my $error = Net::Twitter::Error->new(http_response => $res);
     $error->twitter_error($obj) if ref $obj;
@@ -168,6 +177,51 @@ sub _parse_result {
     die $error;
 }
 
+sub _filter_since {
+    my ($self, $datetime_parser, $obj, $since) = @_;
+
+    # $since can be a DateTime, an epoch value, or a Twitter formatted timestamp
+    my $since_dt  = ref $since && $since->isa('DateTime') && $since
+                 || eval { DateTime->from_epoch(epoch => $since) }
+                 || eval { $datetime_parser->parse_datetime($since) }
+                 || croak
+"Invalid 'since' parameter: $since. Must be a DateTime, epoch, or string in Twitter timestamp format.";
+
+    my $visitor = Data::Visitor::Callback->new(
+        ignore_return_values => 1,
+        array => sub {
+            my ($visitor, $data) = @_;
+
+            return unless $self->_contains_statuses($data);
+
+            # $obj may have inflated created_at attributes, already
+            my $get_created_at_dt = ref $data->[0]{created_at} ? sub { shift->{created_at} }
+                                  : sub { $datetime_parser->parse_datetime(shift->{created_at}) };
+
+            # filter out statuses that are too old
+            my $i = 0;
+            while ( $i < @$data ) {
+                unless ( $get_created_at_dt->($data->[$i]) > $since_dt ) {
+                    splice @$data, $i, 1;
+                }
+                else {
+                    $i++;
+                }
+            }
+        }
+    );
+
+    $visitor->visit($obj);
+}
+
+# check an arrayref to see if it contains satuses
+sub _contains_statuses {
+    my ($self, $arrayref) = @_;
+
+    my $e = $arrayref->[0] || return;
+    return $e->{created_at} && $e->{text} && $e->{id};
+}
+
 1;
 
 __END__
